module StatisticalMethods
  module SummaryStatistic
    module Location
      class Average
        def initialize(array)
          @array = array
          @number = @array.size.to_f
          @empty = @number < 1
        end

        # http://en.wikipedia.org/wiki/Arithmetic_mean
        def arithmetic_mean
          process { @array.addition / @number }
        end

        # http://en.wikipedia.org/wiki/Geometric_mean
        def geometric_mean
          process { @array.multiplication**(1.0 / @number) }
        end

        # http://en.wikipedia.org/wiki/Harmonic_mean
        def harmonic_mean
          process { @number / @array.harmonic.addition } unless include_zero?
        end

        # http://en.wikipedia.org/wiki/Root_mean_square
        def quadratic_mean
          process { Math.sqrt(@array.power(2).addition / @number) }
        end

        # http://en.wikipedia.org/wiki/Generalized_mean
        def power_mean(exponent)
          process do
            return exception(exponent) if exceptions.include? exponent
            (@array.power(exponent).addition / @number)**(1.0 / exponent)
          end
        end

        private

        def process(&block)
          block.call unless @empty
        end

        def include_zero?
          @array.include?(0.0)
        end

        def exceptions
          [-Float::INFINITY, 0, +Float::INFINITY]
        end

        def exception(exponent)
          case exponent
          when 0 then geometric_mean
          when -Float::INFINITY then @array.min
          when +Float::INFINITY then @array.max
          end
        end
      end
    end
  end
end
